import numpy as np
from scipy.misc import imread, imsave
from matplotlib.pyplot import imshow,close,figure



def loadCalibration(filename):

    fid = open(filename, "r")

    # lit la calibration
    v2c = np.vstack(( readVariable(fid, 'Tr_velo_to_cam',3,4), [0,0,0,1])) ;
    p02 = readVariable(fid, 'P2',3,4); 
    rect = np.eye(4) ;
    rect[0:3,0:3] =  readVariable(fid, 'R0_rect',3,3); 

    fid.close();

    return [v2c, rect, p02]




def readVariable(fid,name,M,N):
    # rewind
    fid.seek(0);

    success = 1;
    while success>0:
        line = fid.readline();
        if name in line:
            break;
    if not(success):
        A = [];
    else:
        A = [float(val) for val in line.split()[1:]]
        A = np.reshape(np.array(A),[M,N])
    return A


def plotVeloPointsOnImage(veloPoints, imageFilename, calibrationFilename, newImageFileName, a = 0, b = 0, c = 0, eps = 0):
    ## Cette fonction affiche en rouge les points contenus dans "velopoints", qui sont à une distance epsilon du plan de coordonnées ax+by+z+c=0
    ## elle se base sur l'image imageFilename, à partir du fichier de calibration contenu dans calibrationFilename
    ## le résultat est affiché dans l'image NewImageFilename
    
    
    #charge la matrice de calibration
    [Tr_velo_to_cam, Tr_cam_to_rect, Tr_project_cam_to_plane] = loadCalibration(calibrationFilename)
    img = imread(imageFilename)
    nrPoints = np.shape(veloPoints)[0]
    print("Number of Points Considered: "+np.str(veloPoints.shape[0]))
    #format les points pour la transformation
    veloPointsStack = np.hstack((veloPoints,np.ones((nrPoints,1))))
    # coordonnées des caméras
    camCoords = np.dot(Tr_velo_to_cam ,veloPointsStack.T);
    # rectification
    rectCoords = np.dot(Tr_cam_to_rect,camCoords);
    # projection
    projPointTmp = np.dot(Tr_project_cam_to_plane,rectCoords);
    dist = projPointTmp[2];
    x = np.round(projPointTmp[0] / dist).astype(int);
    y = np.round(projPointTmp[1] / dist).astype(int);
    # vérifie si les pixels sont dans l'image
    idxInsideImage = (x>=0) * (x< img.shape[1]) * (y >=0) * (y <img.shape[0])
    xInsideImage = x[idxInsideImage]
    yInsideImage = y[idxInsideImage]
    img[yInsideImage,xInsideImage,:] = 0
    #affiche les points si distance au plan inférieur à epsilon
    if not((a==0) and (b == 0) and (c == 0) and (eps ==0)):
        dist = getDistancePointsPlan(veloPoints,a,b,c);
        idxCloseToPlan = idxInsideImage * (dist<eps)
        print("Proportion of points with distance less than " +np.str(eps) +": "+np.str(np.sum(idxCloseToPlan)/np.sum(idxInsideImage)))
        xCloseToPlan = x[idxCloseToPlan]
        yCloseToPlan = y[idxCloseToPlan]
        #colorise les points en rouge
        img[yCloseToPlan,xCloseToPlan,0] = 255 
    imsave(newImageFileName,img)
    figure()
    imshow(img)
    close()
    

def getDistancePointsPlan(points,a,b,c):
    d = np.abs(a*points[:,0]+b*points[:,1]+points[:,2]+c)
    return d/np.sqrt(a**2+b**2+1)
